package br.ufpe.rise.travelagency.coreagency.entity.common;

import br.ufpe.rise.travelagency.coreagency.entity.common.AbstractCPRFValidator.EnumCPRF;
import br.ufpe.rise.travelagency.coreagency.entity.customer.CPF;
import br.ufpe.rise.travelagency.coreagency.entity.util.Filler;

public abstract class AbstractCPRF {

	private String formattedCode;
	private Long code;
	protected AbstractCPRFValidator cpAuthenticator;

	public static AbstractCPRF create(Long cadastroDePessoa, EnumCPRF tipoDeCadastro) 
				throws IllegalArgumentException {

		return create(String.valueOf(cadastroDePessoa),tipoDeCadastro);
	}

	public static AbstractCPRF create(String cadastroDePessoa, EnumCPRF tipoDeCadastro) 
				throws IllegalArgumentException {
		
		String codigo = null;
		
		if (AbstractCPRFValidator.isValidParameters(cadastroDePessoa, tipoDeCadastro)) {

			switch_Cadastro: {

				if (tipoDeCadastro == EnumCPRF.CPF) {

					codigo = Filler.ZERO_LEFT.fill(cadastroDePessoa, 11);
					
					break switch_Cadastro;
				}
			}
		}
		
		return create(codigo);
	}
	
	public static AbstractCPRF create(String personRegister)
			throws IllegalArgumentException {

		AbstractCPRF cp = null;
		AbstractCPRFValidator cpAuthenticator = AbstractCPRFValidator.create(personRegister);

		if (cpAuthenticator.isValid()) {

			if (cpAuthenticator.isPhisyc()) {

				cp = new CPF(cpAuthenticator.getRegisterCode());

			} 
			
			cp.cpAuthenticator = cpAuthenticator;

		} else {

			throw new IllegalArgumentException(
					"The person register [ "+personRegister+" ] não é válido.");
		}

		return cp;
	}
	
	public boolean isPhysic(){
		return cpAuthenticator.isPhisyc();
	}

	protected String getFormattedCode() {
		return formattedCode;
	}

	protected void setFormattedCode(String formattedCode) {
		this.formattedCode = formattedCode;
	}

	public Long getCode() {
		return code;
	}

	public void setCode(Long code) {
		this.code = code;
	}
}