package br.ufpe.rise.travelagency.coreagency.entity.common;

import static br.ufpe.rise.travelagency.coreagency.entity.util.ObjectUtil.isNotNull;

import java.io.Serializable;
import java.util.regex.Pattern;

import org.apache.commons.lang.StringUtils;

import br.ufpe.rise.travelagency.coreagency.entity.util.ObjectUtil;

public abstract class AbstractCPRFValidator {
	
	private static final String REGEX_CPF = "(\\d{3}\\.\\d{3}\\.\\d{3}\\-\\d{2})|(\\d{11})";
	private static final String REGEX_REGISTER = "\\d{3,14}";
	
	protected StringBuilder registerCode;
	protected AbstractDigitValidator digitValidator;

	public enum EnumCPRF implements Serializable {

		CPF;

		public AbstractCPRFValidator getAuthenticator() {

			AbstractCPRFValidator validator = null;

			switch (this) {
			
			case CPF:
				
				validator = new CPFValidator();
				validator.digitValidator = new CPFDV();
				
				break;
			}

			return validator;
		}
		
		public String toString(){
			return ObjectUtil.toString(this);
		}
	}

	private EnumCPRF registerType;

	public abstract boolean isValid();
	
	protected abstract void removeFormat();

	public static AbstractCPRFValidator create(
			String registerCode) throws IllegalArgumentException {

		AbstractCPRFValidator validator_A_CP = null;

		validator_A_CP = create(selectRegisterType(registerCode));	
		
		validator_A_CP.registerCode = new StringBuilder (registerCode);
		validator_A_CP.removeFormat();

		return validator_A_CP;
	}

	public static AbstractCPRFValidator create(EnumCPRF registerType){
		
		AbstractCPRFValidator validator_A_CP = null;

		if(isNotNull(registerType)){
			
			validator_A_CP = registerType.getAuthenticator();
			
			validator_A_CP.registerType = registerType;
			
		}else
			throw new IllegalArgumentException(
					"Null register type: [ "+registerType+" ]");
		
		return validator_A_CP;
	}
	
	private static EnumCPRF selectRegisterType(
			String registerCode) throws IllegalArgumentException {

		EnumCPRF type = null;

		switch_Type: {

			if (StringUtils.isNotBlank(registerCode)) {

				if (Pattern.matches(REGEX_CPF, registerCode)) {

					type = EnumCPRF.CPF;

					break switch_Type;
					
				}
				
			}
			
			throw new IllegalArgumentException(
					"The register code [ "+ registerCode +" ] is not on a valid format!");
		}

		return type;
	}

	public void setRegisterCode(String registerCode) {

		if (selectRegisterType(registerCode) == this.registerType) {

			this.registerCode.delete(0, registerCode.length());
			
			this.registerCode.append(registerCode);
			
			removeFormat();
		} else
			throw new IllegalArgumentException("This is a validator: "
					+ this.registerType);

	}

	public static boolean isValidParameters(String registerCode,
			EnumCPRF registerType) throws IllegalArgumentException {

		boolean isValid = false;

		if (isNotNull(registerCode) && isNotNull(registerType)) {

			if (Pattern.matches(REGEX_REGISTER, registerCode)) {

				isValid = true;

			} else {

				throw new IllegalArgumentException(
						"The register is wrong or doesn't exists: [ "
								+ registerCode + " ]");
			}
		} else {

			throw new IllegalArgumentException(
					"O tipo de cadastro está incorreto: [ " + registerType
							+ " ] ou o cadastro não exsite: [ "
							+ registerCode + " ]");
		}

		return isValid;
	}
	
	public String getRegisterCode() {

		return registerCode.toString();
	}

	public boolean isPhisyc() {

		return this instanceof CPFValidator;
	}
}