package br.ufpe.rise.travelagency.coreagency.entity.common;

import java.util.regex.Pattern;

import org.apache.commons.lang.StringUtils;

import br.ufpe.rise.travelagency.coreagency.entity.util.Filler;
import br.ufpe.rise.travelagency.coreagency.entity.util.Module;
import br.ufpe.rise.travelagency.coreagency.entity.util.ModuleType;

public class CPFDV extends AbstractDigitValidator {
	
	private static final int LIMITE_MINIMO = 2;
	private static final String REGEX_CPF_DV = "\\d{9}";
	private static final String REGEX_CPF_DV_FORMATED = "\\d{3}\\.\\d{3}\\.\\d{3}";

	@Override
	public int calculate(long number) {
		
		return calculate(Filler.ZERO_LEFT.fill(String.valueOf(number), 9));
	}

	@Override
	public int calculate(String number) throws IllegalArgumentException {

		int dv1 = 0;
		int dv2 = 0;
		boolean isValidFormat = false;

		validation: {

			if (StringUtils.isNotBlank(number)) {

				isValidFormat = (Pattern.matches(REGEX_CPF_DV, number) || Pattern
						.matches(REGEX_CPF_DV_FORMATED, number));

				if (isValidFormat) {

					number = StringUtils.replaceChars(number, ".", "");

					dv1 = calculate(number, 10);
					dv2 = calculate(number + dv1, 11);

					break validation;
				}
			}

			throw new IllegalArgumentException(
					"The CPF [ "+number+" ] must only contains numbers, " +
							"and it must be in the following format " +
							"###.###.### ou ######### !");

		}

		return Integer.parseInt(dv1 + "" + dv2);

	}
	
	private int calculate(String number, int moduleMaxLimit)
			throws IllegalArgumentException {

		int dv = 0;
		int rest = 0;
		
		rest = Module.calculateMod11(number, LIMITE_MINIMO, moduleMaxLimit);

		if (rest >= 2) {

			dv = ModuleType.MODULE11.value() - rest;
		}

		return dv;
	}
}