package br.ufpe.rise.travelagency.coreagency.entity.customer;

import br.ufpe.rise.travelagency.coreagency.entity.common.AbstractCPRF;
import br.ufpe.rise.travelagency.coreagency.entity.common.AbstractCPRFValidator;
import br.ufpe.rise.travelagency.coreagency.entity.common.AbstractCPRFValidator.EnumCPRF;
import br.ufpe.rise.travelagency.coreagency.entity.util.Filler;
import br.ufpe.rise.travelagency.coreagency.exception.CPFException;

public class CPF extends AbstractCPRF implements ICPF {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 7588209194247235742L;
	
	private String cpf;
	
	public CPF(String strCPF) {
		
		this.cpf = strCPF;
		
		this.cpAuthenticator = AbstractCPRFValidator.create(this.cpf);

		if (cpAuthenticator.isValid()) {
			init(this.cpf);
		} else {
			try {
				throw new CPFException(new IllegalArgumentException(
						"The person register [ " + this.cpf + " ] is not valid."));
			} catch (CPFException e) {
				e.printStackTrace();
			}
		}
	}

	public CPF(Long numCPF) {

		try {

			if (AbstractCPRFValidator.isValidParameters(String.valueOf(numCPF),
					EnumCPRF.CPF)) {

				String strCPF = Filler.ZERO_LEFT.fill(numCPF, 14);
				
				this.cpf = strCPF;
				
				this.cpAuthenticator = AbstractCPRFValidator.create(this.cpf);

				if (cpAuthenticator.isValid())
					init(this.cpf);
				else
					throw new IllegalArgumentException(
							"The person register [ " + this.cpf
									+ " ] is not valid.");
			}

		} catch (Exception e) {
			if (!(e instanceof CPFException))
				try {
					throw new CPFException(e);
				} catch (CPFException e1) {
					e1.printStackTrace();
				}
		}
	}

	public String getCpf() {
		return cpf;
	}

	private void init(String strCPF) {

		try {

			StringBuilder formattedCode = null;

			formattedCode = new StringBuilder(strCPF);

			formattedCode.insert(3, '.');
			formattedCode.insert(7, '.');
			formattedCode.insert(11, '-');

			this.setFormattedCode(formattedCode.toString());
			this.setCode(Long.parseLong(strCPF));

		} catch (Exception e) {
			try {
				throw new CPFException(e);
			} catch (CPFException e1) {
				e1.printStackTrace();
			}
		}
	}
}