package br.ufpe.rise.travelagency.creditcard.component.impl;

import br.ufpe.rise.travelagency.coreagency.entity.util.Validator;
import br.ufpe.rise.travelagency.creditcard.component.ICreditCardComponent;
import br.ufpe.rise.travelagency.creditcard.entity.CreditCard;

public class CreditCardComponentImpl implements ICreditCardComponent {
	
	private CreditCard card;
	
	public CreditCardComponentImpl(CreditCard card){
		this.card = card;
	}
	
	public CreditCard getCard(){
		return card;
	}

	public boolean isCreditCard(){
		String cardValue = this.card.getCreditCardNumberWithoutPunctuation();
        if (cardValue == null) {
            return false;
        }

        //Strip out non-numeric characters from the card string
        StringBuffer numberToValidate = new StringBuffer();
        
        for (int i = 0; i < cardValue.length(); i++) {
            if (Character.isDigit(cardValue.charAt(i))) {
                numberToValidate.append(cardValue.charAt(i));
            }
        }

        return validateCreditCardLengthAndPrefix(numberToValidate.toString()) &&
            validateCreditCardWithLuhn(numberToValidate.toString());
    }

    private boolean validateCreditCardLengthAndPrefix(String cardNumber) {
        boolean isValid = false;
        int length = cardNumber.length();
        byte secondChar;

        if (length >= 13) {
        	
            switch (cardNumber.charAt(0)) {
                case '3':  //American express
                    secondChar = Byte.parseByte(cardNumber.substring(1,2));
                    if (length == 15 && (secondChar == 4 || secondChar == 7)) {
                        isValid = true;
                    }
                    break;
                case '4':  //Visa
                    if (cardNumber.length() == 13 || cardNumber.length() == 16) {
                        isValid = true;
                    }
                    break;
                case '5':  //Mastercard
                    secondChar = Byte.parseByte(cardNumber.substring(1,2));
                    if (length == 16 && Validator.isInRange(secondChar, 1, 5)) {
                        isValid = true;
                    }
                    break;
                case '6':  //Discover
                    if (cardNumber.startsWith("6011") && length == 16) {
                        isValid = true;
                    }
                    break;
            }
        }

        return isValid;
    }

    private boolean validateCreditCardWithLuhn(String cardNumber) {
        // number must be validated as 0..9 numeric first!!
        int no_digit = cardNumber.length();
        int oddoeven = no_digit & 1;
        long sum = 0;
        for (int count = 0; count < no_digit; count++) {
           int digit = 0;
           try {
              digit = Integer.parseInt(String.valueOf(cardNumber.charAt(count)));
           } catch (NumberFormatException e) {
              return false;
           }
           if (((count & 1) ^ oddoeven) == 0) { // not
               digit *= 2;
               if (digit > 9) {
                  digit -= 9;
               }
           }
           sum += digit;
        }
        
        if (sum == 0) {
           return false;
        }

        if (sum % 10 == 0) {
           return true;
        }

        return false;
    }
}