package br.ufpe.rise.travelagency.creditcard.entity;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import br.ufpe.rise.travelagency.coreagency.entity.util.Validator;

public abstract class CreditCardType {

	@SuppressWarnings("unchecked")
	private static List types = new ArrayList();
	   
    public static final CreditCardType VISA =
        new CreditCardType("10", "prm.global.creditcard.types.visa",
        new Object[] {new CreditCardFormat("####-####-####-####", 16),
        new CreditCardFormat("#####-####-####", 13)}) {
    	
            public boolean numberMatchesType(String ccNumber) {
                if (ccNumber.length() != 13 && ccNumber.length() != 16) {
                    return false;
                }
                
                if (!ccNumber.startsWith("4")) {
                    return false;
                }
                return true;
            }
        };
   
    public static final CreditCardType MASTERCARD =
        new CreditCardType("20", "prm.global.creditcard.types.mastercard",
        new Object [] {new CreditCardFormat("####-####-####-####",16)}) {
    	
            public boolean numberMatchesType(String ccNumber) {
                if (ccNumber.length() < 16) {
                    return false;
                }
                
                byte firstChar = Byte.parseByte(ccNumber.substring(0,1));
                byte secondChar = Byte.parseByte(ccNumber.substring(1,2));
                
                if (firstChar == 5 && Validator.isInRange(secondChar, 1, 5))
                    return true;
                else
                    return false;
            }
    	};
    
    public static final CreditCardType AMEX =
        new CreditCardType("30", "prm.global.creditcard.types.amex",
        new Object[] {new CreditCardFormat("####-######-#####", 15)}) {
            public boolean numberMatchesType(String ccNumber) {
                return (ccNumber.startsWith("33") || ccNumber.startsWith("37"))
                    && ccNumber.length() == 15;
            }
        };
  
    public static final CreditCardType DEFAULT = VISA;
    
    @SuppressWarnings("unchecked")
	public static CreditCardType getForID(String id) {
        CreditCardType toReturn = DEFAULT;

        for (Iterator it = types.iterator(); it.hasNext();) {
            CreditCardType type = (CreditCardType)it.next();
            if (type.getID().equals(id)) {
                toReturn = type;
                break;
            }
        }

        return toReturn;
    }

    private String id;
    @SuppressWarnings("unused")
	private String displayToken;
    @SuppressWarnings("unchecked")
	private Map displayFormats = new HashMap();

    @SuppressWarnings("unchecked")
	private CreditCardType(String id, String displayToken, Object[] displayFormats) {
        this.id = id;
        this.displayToken = displayToken;
        types.add(this);

        for (int i = 0; i < displayFormats.length; i++) {
            this.displayFormats.put(
                new CreditCardLookupKey(this, ((CreditCardFormat)displayFormats[i]).cardLength),
                ((CreditCardFormat)displayFormats[i]).cardFormat);
        }
    }

    public String getID() {
        return id;
    }

    public String getHtmlOptionValue() {
        return getID();
    }

    public String getHtmlOptionDisplay() {
        return toString();
    }

    public abstract boolean numberMatchesType(String ccNumber);

    public String formatNumberString(String creditCardNumber) {
        StringBuffer formattedCardNumber = new StringBuffer();

        //Find the correct credit card format
        String creditCardFormat = (String)displayFormats.get(
            new CreditCardLookupKey(this, creditCardNumber.length()));

        //If we didn't find a format, use a default
        if (creditCardFormat == null) {
            char[] defaultFormat = new char[creditCardNumber.length()];
            Arrays.fill(defaultFormat, '#');
            creditCardFormat = new String(defaultFormat);
        }

        int creditCardNumberIndex = 0;
        for (int i = 0; i < creditCardFormat.length(); i++) {
            if (creditCardFormat.charAt(i) == '#') {
                formattedCardNumber.append(creditCardNumber.charAt(creditCardNumberIndex++));
            } else if (creditCardFormat.charAt(i) == 'X') {
                formattedCardNumber.append('X');
                creditCardNumberIndex+=1;
            } else {
                formattedCardNumber.append(creditCardFormat.charAt(i));
            }
        }

        return formattedCardNumber.toString();
    }
}

class CreditCardFormat {

    String cardFormat;
    int cardLength;

    public CreditCardFormat(String cardFormat, int cardLength) {
        this.cardFormat = cardFormat;
        this.cardLength = cardLength;
    }
}

/**
 * Class which provides the ability to look up a credit card format according to
 * the card type and length.
 */
class CreditCardLookupKey {
   
    private CreditCardType type;
    
    /** Number of digits in the credit card. */
    private int length;

    public CreditCardLookupKey(CreditCardType type, int length) {
        this.type = type;
        this.length = length;
    }

    public boolean equals(Object o) {
        if (this == o)
            return true;
        if (!(o instanceof CreditCardLookupKey))
            return false;

        final CreditCardLookupKey creditCardLookupKey = (CreditCardLookupKey)o;

        if (length != creditCardLookupKey.length)
            return false;
        if (type != null ? !type.equals(creditCardLookupKey.type) : creditCardLookupKey.type != null)
            return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = (type != null ? type.hashCode() : 0);
        result = 29 * result + length;
        return result;
    }
}