package br.com.rise.data.access.hibernate;

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Criterion;

import br.com.rise.data.access.exception.DataAccessLayerException;
import br.com.rise.data.access.service.HibernateFactory;

/**
 * 
 * @author heberthbraga
 *
 * @param <Entity>
 */
public class DataAccessHibernate<Entity> implements IDataAccessGeneric<Entity> {
	
	private Class<Entity> type;
	private List<String> files;
	private Session session;
	private Transaction tx;
	private HibernateFactory factory;
	
	public DataAccessHibernate(){

	}
	
	public void setType(Class<Entity> type) {
		this.type = type;
	}
	
	public Class<Entity> getType(){
	
		return this.type;
	}
	
	public void setFiles(List<String> files){
		this.files = files;
	}
	
	public List<String> getFiles(){
		
		return this.files;
	}

	@Override
	public void delete(Entity entity) {
		try {
			startOperation(this.files);
			session.delete(entity);
			tx.commit();
		} catch (HibernateException e) {
			e.printStackTrace();
			tx.rollback();
		} catch (DataAccessLayerException e) {
			e.printStackTrace();
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public Entity getById(Object identifier) {
		Entity t = null;
		
		try {
			startOperation(this.files);
			t = (Entity) session.get(this.type, (Serializable) identifier);
			tx.commit();
		} catch (HibernateException e) {
			e.printStackTrace();
			tx.rollback();
		} catch (DataAccessLayerException e) {
			e.printStackTrace();
		} 
		
		return t;
	}

	@Override
	public List<Entity> listAll() {
	
		return findByCriteria();
	}

	@Override
	public Entity persist(Entity entity) {
		try {
			startOperation(this.files);
			session.saveOrUpdate(entity);
			session.flush();
			tx.commit();
		} catch (HibernateException e) {
			e.printStackTrace();
			tx.rollback();
		} catch (DataAccessLayerException e) {
			e.printStackTrace();
		}
		
		return entity;
	}

	@Override
	public void update(Entity entity) {
		try {
			startOperation(this.files);
			session.update(entity);
			tx.commit();
		} catch (HibernateException e) {
			e.printStackTrace();
			tx.rollback();
		} catch (DataAccessLayerException e) {
			e.printStackTrace();
		}
	}
	
	@SuppressWarnings("unchecked")
	private List<Entity> findByCriteria(Criterion... criterion) {
		List<Entity> list = null;
		
		try {
			startOperation(this.files);
			
			Criteria criteria = session.createCriteria(type);
			
	        for (Criterion c : criterion) {
	        	criteria.add(c);
	        }
	        
	        list = criteria.list();
	        
			tx.commit();
		} catch (HibernateException e) {
			e.printStackTrace();
			tx.rollback();
		} catch (DataAccessLayerException e) {
			e.printStackTrace();
		}
		
        return list;
	}
	
	@SuppressWarnings("unchecked")
	public Collection<Object> find(String search, Collection<Object> parameters){
		Collection<Object> list = null;
		
		try {
			startOperation(this.files);
	
			Query query = session.createQuery(search);
			
			int i = 0;
			Iterator<Object> itParametros = parameters.iterator();
			
			while(itParametros.hasNext()) {
				Object param = itParametros.next();
				String strParam = "param" + i;
				
				if (param instanceof Integer) {
					query.setInteger(strParam, ((Integer) param));
				} else if (param instanceof Double) {
					query.setDouble(strParam, ((Double) param));
				} else if (param instanceof Long) {
					query.setLong(strParam, ((Long) param));
				} else if (param instanceof String) {
					query.setString(strParam, ((String) param));
				} else if (param instanceof Boolean) {
					query.setBoolean(strParam, ((Boolean) param));
				}
				i++;
			}
		
			list = query.list();
			
			tx.commit();
			
		} catch (HibernateException e) {
			e.printStackTrace();
			tx.rollback();
		} catch (DataAccessLayerException e) {
			e.printStackTrace();
		}
		
		return list;
	}
	
	public Session getCurrentSession(){
		
		return session;
	}
	
	public Transaction getTransaction(){
		
		return tx;
	}
	
	public void startOperation(List<String> files) throws HibernateException, DataAccessLayerException{
		factory = new HibernateFactory(files);
		session = factory.getSessionFactory().getCurrentSession();
		tx = session.beginTransaction();
	}
}