package br.com.rise.data.access.service;

import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.Configuration;

import br.com.rise.data.access.exception.DataAccessLayerException;

/**
 * 
 * @author heberthbraga
 *
 */
public class HibernateFactory {

	private SessionFactory sessionFactory;
	
	public HibernateFactory(List<String> files){
		this.sessionFactory = buildSessionFactory(files);
	}
	
	public  SessionFactory buildSessionFactory(List<String> files) 
		throws HibernateException {
		if (this.sessionFactory != null) {
			closeFactory();
		}
		
		return configureSessionFactory(files);
	}
	
	public SessionFactory getSessionFactory() {
        return this.sessionFactory;
    }
    
    public Session openSession() throws HibernateException, DataAccessLayerException {
   
        return this.sessionFactory.openSession();
    }

	public void closeFactory() {
		if (this.sessionFactory != null) {
			try {
				this.sessionFactory.close();
			} catch (HibernateException ignored) {
				ignored.printStackTrace();
			}
		}
	}
	
	public void close(Session session) {
		if (session != null) {
			try {
				session.close();
			} catch (HibernateException ignored) {
				ignored.printStackTrace();
			}
		}
	}

	public void rollback(Transaction tx) {
		try {
			if (tx != null) {
				tx.rollback();
			}
		} catch (HibernateException ignored) {
			ignored.printStackTrace();
		}
	}
	
	/**
	 *
	 * @return
	 * @throws HibernateException
	 */
	private SessionFactory configureSessionFactory(List<String> files) throws HibernateException {
		
		Configuration configuration = new Configuration();
		configuration.configure();
		
		for(String file : files){
			configuration.addResource(file);
		}
		
		this.sessionFactory = configuration.buildSessionFactory();
		
		return this.sessionFactory;
	}
}