package com.hotel.common.service.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;

import com.hotel.common.entity.Booking;
import com.hotel.common.entity.IBooking;
import com.hotel.common.entity.IHotel;
import com.hotel.common.exception.BookingException;
import com.hotel.common.service.IHotelService;

public class HotelService implements IHotelService {

	private IHotel hotel;
	private IBooking booking;
	private Object user;
	private boolean bookingValid;
	private Collection<IBooking> bookings;
	
	public HotelService(){
		bookings = new ArrayList<IBooking>();
	}
	
	@Override
	public void bookHotel() {
		booking = new Booking(hotel, user);
		Calendar calendar = Calendar.getInstance();
		booking.setCheckinDate( calendar.getTime() );
		calendar.add(Calendar.DAY_OF_MONTH, 1);
		booking.setCheckoutDate( calendar.getTime() );
	}

	@Override
	public IBooking getBooking() {
		
		return booking;
	}
	
	@Override
	public void addBooking(IBooking booking){
		if(!bookings.contains(booking) || bookings.isEmpty()){
			bookings.add(booking);
		}
	}
	
	@Override
	public void removeBooking(IBooking booking){
		if(bookings.contains(booking)){
			bookings.remove(booking);
		}
	}

	@Override
	public Collection<IBooking> getBookings() {
		return bookings;
	}

	@Override
	public boolean isBookingValid() {
		
		return bookingValid;
	}

	@Override
	public void setBookingDetails() throws BookingException {
		Calendar calendar = Calendar.getInstance();
		calendar.add(Calendar.DAY_OF_MONTH, -1);
		
		if ( booking.getCheckinDate().before
				( calendar.getTime() ) ){
			
			bookingValid = false;
			throw new BookingException("Check in date must be a future date");
			
		}else if ( !booking.getCheckinDate().before
				( booking.getCheckoutDate() ) ){
			bookingValid=false;
			throw new BookingException("Check out date must be later than check in date");
			
		}else {
			bookingValid=true;
		}
	}
}