package br.ufpe.rise.travelagency.service.impl;

import java.util.List;

import br.ufpe.rise.travelagency.bank.IAccount;
import br.ufpe.rise.travelagency.billet.component.IBilletComponent;
import br.ufpe.rise.travelagency.billet.entity.BilletBankType;
import br.ufpe.rise.travelagency.billet.entity.IBillet;
import br.ufpe.rise.travelagency.component.IPaymentComponent;
import br.ufpe.rise.travelagency.creditcard.component.ICreditCardComponent;
import br.ufpe.rise.travelagency.creditcard.entity.ICreditCard;
import br.ufpe.rise.travelagency.payment.IPayment;
import br.ufpe.rise.travelagency.service.IPaymentService;

public class PaymentService implements IPaymentService {
	
	private IBilletComponent billetComponent;
	private ICreditCardComponent card;
	private IPaymentComponent paymentComponent;
	private IBillet billet;
	
	public PaymentService(){
		billet = null;
	}
	
	//Injecting the billet component variant
	public void setBilletComponent(IBilletComponent billetComponent) {
		this.billetComponent = billetComponent;
	}
	
	//Injecting the credit card component variant
	public void setCard(ICreditCardComponent card) {
		this.card = card;
	}
	
	public void setPaymentComponent(IPaymentComponent paymentComponent) {
		this.paymentComponent = paymentComponent;
	}

	public boolean verifyCreditCard(){
		return this.card.isCreditCard();
	}
	
	public ICreditCard createCreditCard(){
		return this.card.getCard();
	}

	public void generateBillet(int bankCode, String inputFile){
		this.billetComponent.generateBillet(bankCode, inputFile);
	}
	
	public IBillet createBillet(BilletBankType type){
		
		this.billetComponent.createBillet(billet);
		
		billet.setType(type);
		
		this.billetComponent.createBillet(billet);
		
		return this.billetComponent.getBillet();
	}

	@Override
	public void addAccount(IAccount account) {
		this.paymentComponent.save(account);
	}

	@Override
	public void addPayment(IPayment payment) {
		this.paymentComponent.save(payment);
	}

	@Override
	public IAccount getAccountById(Long id) {
		
		return this.paymentComponent.getAccountById(id);
	}

	@Override
	public List<IAccount> getAccounts() {
		
		return this.paymentComponent.getAccounts();
	}

	@Override
	public List<IPayment> getPayments() {
		
		return this.paymentComponent.getPayments();
	}
}