package br.ufpe.rise.travelagency.coreagency.entity.util;

import static br.ufpe.rise.travelagency.coreagency.entity.util.ObjectUtil.isNotNull;

import org.apache.commons.lang.StringUtils;

public class Filler<G> {
	
	public static final Filler<Integer> ZERO_LEFT = new Filler<Integer>(0, SideToFill.LEFT);
	public static final Filler<Integer> ZERO_RIGHT = new Filler<Integer>(0, SideToFill.RIGHT);
	public static final Filler<String> WHITE_SPACE_LEFT = new Filler<String>(StringUtil.WHITE_SPACE, SideToFill.LEFT);
	public static final Filler<String> WHITE_SPACE_RIGHT = new Filler<String>(StringUtil.WHITE_SPACE, SideToFill.RIGHT);

	private G fillWith;
	
	private SideToFill sideToFill;
	
	public Filler(G fillWith, SideToFill sideToFill) {
		
		setFillWith(fillWith);
		setSideToFill(sideToFill);
	}
	
	public G getFillWith() {
		return fillWith;
	}

	public void setFillWith(G fillWith) {
		
		if(isNotNull(fillWith))
			this.fillWith = fillWith;
		
		else
			throw new IllegalArgumentException("Filler inválido [ " + fillWith + " ]!");
	}

	public SideToFill getSideToFill() {
		return sideToFill;
	}

	public void setSideToFill(SideToFill sideToFill) {
		
		if(isNotNull(sideToFill))
			this.sideToFill = sideToFill;
		
		else
			throw new IllegalArgumentException("Lado inválido [ " + sideToFill + " ]!");
	}
	
	public String fill(String toFill, int length){
		
		String str = null;
		
		switch(sideToFill){
		
			case LEFT:
				str = fillLeft(toFill, length);
				break;
				
			case RIGHT:
				str = fillRight(toFill, length);
				break;
		}
		
		return str;
	}
	
	public String fill(long toFill, int length){
		return fill(String.valueOf(toFill), length);
	}
	
	public String fill(int toFill, int length){
		return fill(String.valueOf(toFill), length);
	}
	
	public String fill(short toFill, int length){
		return fill(String.valueOf(toFill), length);
	}
	
	public String fill(byte toFill, int length){
		return fill(String.valueOf(toFill), length);
	}
	
	public String fill(char toFill, int length){
		return fill(String.valueOf(toFill), length);
	}
	
	public String fill(double toFill, int length){
		return fill(String.valueOf(toFill), length);
	}
	
	public String fill(float toFill, int length){
		return fill(String.valueOf(toFill), length);
	}
	
	public String fill(Object toFill, int length){
		
		String toFillTemp = null;
		
		if (isNotNull(toFill)) {
			toFillTemp = toFill.toString();
		}
		
		return fill(toFillTemp, length);
	}
	
	public String fill(TextStream toFill, int length){

		String toFillTemp = null;
		
		if(isNotNull(toFill))
			toFillTemp = toFill.write();
		
		return fill(toFillTemp, length);
	}
	
	private String fillRight(String toFill, int length) {
		
		return StringUtils.rightPad(toFill, length, fillWith.toString());
	}

	private String fillLeft(String toFill, int length) {
		
		return StringUtils.leftPad(toFill, length, fillWith.toString());
	}

	public enum SideToFill{
		
		LEFT,
		RIGHT;
	}

	@Override
	public String toString() {
		return ObjectUtil.toString(this);
	}
}