package br.ufpe.rise.travelagency.coreagency.entity.util;

import static br.ufpe.rise.travelagency.coreagency.entity.util.ObjectUtil.isNotNull;

import org.apache.commons.lang.StringUtils;

public class Module {

	private static final String THE_ARGUMENT_MUST_ONLY_CONTAIN_NUMBERS = 
		"The arguments must only contains numbers!";
	
	public static final int MOD10 = ModuleType.MODULE10.value();
	public static final int MOD11 = ModuleType.MODULE11.value();
	
	private ModuleType mod;
	private int maxLimit;
	private int minLimit;

	public Module(ModuleType mod) {
		super();

		if (isNotNull(mod, "module")) {

			this.mod = mod;
			initDefault();
		}
	}

	public Module(ModuleType mod, int maxLimit, int minLimit) {
		super();

		if (isNotNull(mod, "modulo")) {

			this.maxLimit = maxLimit;
			this.minLimit = minLimit;
			this.mod = mod;
		}
	}

	public int value() {

		switch (mod) {

		case MODULE10:

			return ModuleType.MODULE10.value();

		case MODULE11:

			return ModuleType.MODULE11.value();

		default:
			return -1;
		}

	}
	
	public static int calculateMod11(long number, int minLimit, int maxLimit) {

		return calculateMod11(String.valueOf(number), minLimit, maxLimit);
	}

	public static int calculateMod11(String number, int minLimit, int maxLimit)
			throws IllegalArgumentException {

		return (calculateSequencialSumMod11(number, minLimit, maxLimit) % 11);
	}

	public static int calculateSequencialSumMod11(String number, int minLimit,
			int maxLimit) throws IllegalArgumentException {

		int weight = 0;
		int sum = 0;

		if (StringUtils.isNotBlank(number) && StringUtils.isNumeric(number)) {

			StringBuilder sb = new StringBuilder(number);
			sb.reverse();

			weight = minLimit;

			for (char c : sb.toString().toCharArray()) {
				sum += weight * Character.getNumericValue(c);
				weight++;

				if (weight > maxLimit)
					weight = minLimit;
			}

		} else
			throw new IllegalArgumentException(
					THE_ARGUMENT_MUST_ONLY_CONTAIN_NUMBERS);

		return sum;
	}

	public static int calculateMod10(long number, int minLimit, int maxLimit) {

		return calculateMod10(String.valueOf(number), minLimit, maxLimit);
	}

	public static int calculateMod10(String number, int minLimit, int maxLimit)
			throws IllegalArgumentException {

		return (calculateSequentialSumMod10(number, minLimit, maxLimit) % 10);
	}

	public static int calculateSequentialSumMod10(String number, int minLimit,
			int maxLimit) throws IllegalArgumentException {

		int product = 0;
		int weight = 0;
		int sum = 0;

		if (StringUtils.isNotBlank(number) && StringUtils.isNumeric(number)) {

			StringBuilder sb = new StringBuilder(number);
			sb.reverse();

			weight = maxLimit;

			for (char c : sb.toString().toCharArray()) {

				product = weight * Character.getNumericValue(c);

				if (product > 9) {

					sum += product / 10;
					sum += product % 10;
				} else
					sum += product;

				weight = (weight == maxLimit) ? minLimit : maxLimit;
			}

		} else
			throw new IllegalArgumentException(
					THE_ARGUMENT_MUST_ONLY_CONTAIN_NUMBERS);

		return sum;
	}

	public int calcule(String numero) throws IllegalArgumentException {

		int modulo = 0;

		switch (mod) {

		case MODULE10:

			modulo = calculateMod10(numero, getMaxLimit(), getMaxLimit());

			break;

		case MODULE11:

			modulo = calculateMod11(numero, getMinLimit(), getMinLimit());

			break;
		}

		return modulo;
	}

	public int calculate(long number) {

		return calcule(String.valueOf(number));
	}

	private void initDefault() {

		switch (mod) {

		case MODULE10:

			setMaxLimit(1);
			setMaxLimit(2);

			break;

		case MODULE11:

			setMaxLimit(2);
			setMaxLimit(9);

			break;
		}
	}
	
	public ModuleType getMod() {
		return mod;
	}

	public void setMod(ModuleType mod) {
		this.mod = mod;
	}

	public int getMaxLimit() {
		return maxLimit;
	}

	public void setMaxLimit(int maxLimit) {
		this.maxLimit = maxLimit;
	}

	public int getMinLimit() {
		return minLimit;
	}

	public void setMinLimit(int minLimit) {
		this.minLimit = minLimit;
	}

	@Override
	public String toString() {
		return ObjectUtil.toString(this);
	}
}